package org.warp.jcwdb.tests;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.warp.jcwdb.EnhancedObject;
import org.warp.jcwdb.database.IDatabaseTools;
import org.warp.jcwdb.annotations.DBDataType;
import org.warp.jcwdb.annotations.DBField;
import org.warp.jcwdb.annotations.DBPropertyGetter;
import org.warp.jcwdb.annotations.DBPropertySetter;
import org.warp.jcwdb.utils.NTestUtils;

import java.io.IOException;

public class MultipleEnhancedObjects {
	private NTestUtils.WrappedDb db;
	private RootTwoClasses root;

	@Before
	public void setUp() throws Exception {
		db = NTestUtils.wrapDb().create((db) -> {
			root = db.get().loadRoot(RootTwoClasses.class, RootTwoClasses::new);
		});
		root.class1 = new NTestUtils.RootClass(db.get());
		db.setRootClassValues(root.class1);
		root.class2 = new NTestUtils.RootClass(db.get());
		db.setRootClassValues(root.class2);
		root.setClass3(new NTestUtils.RootClass(db.get()));
		db.setRootClassValues(root.getClass3());
		root.setClass4(new NTestUtils.RootClass(db.get()));
		db.setRootClassValues(root.getClass4());
		db.closeAndReopen();
	}

	@Test
	public void shouldMatchMultipleEnhancedObjects() {
		db.testRootClassValues(root.class1);
		db.testRootClassValues(root.class2);
		db.testRootClassValues(root.getClass3());
		db.testRootClassValues(root.getClass4());
	}

	@After
	public void tearDown() throws Exception {
		db.delete();
	}

	public static class RootTwoClasses extends EnhancedObject {

		@DBField(id = 0, type = DBDataType.ENHANCED_OBJECT)
		public NTestUtils.RootClass class1;

		@DBField(id = 1, type = DBDataType.ENHANCED_OBJECT)
		public NTestUtils.RootClass class2;

		public RootTwoClasses() {
			super();
		}

		public RootTwoClasses(IDatabaseTools databaseTools) throws IOException {
			super(databaseTools);
		}

		@DBPropertyGetter(id = 0, type = DBDataType.ENHANCED_OBJECT)
		public NTestUtils.RootClass getClass3() {
			return getProperty();
		}

		@DBPropertySetter(id = 0, type = DBDataType.ENHANCED_OBJECT)
		public void setClass3(NTestUtils.RootClass value) {
			setProperty(value);
		}

		@DBPropertyGetter(id = 1, type = DBDataType.ENHANCED_OBJECT)
		public NTestUtils.RootClass getClass4() {
			return getProperty();
		}

		@DBPropertySetter(id = 1, type = DBDataType.ENHANCED_OBJECT)
		public void setClass4(NTestUtils.RootClass value) {
			setProperty(value);
		}
	}
}
